//  Equinox Desktop Environment - desktop icons manager
//  Copyright (C) 2000-2001 Martin Pekar
//
//  Requires Fast Light Tool Kit 2.0 (FLTK)
//  http://www.fltk.org
//
//  Author  : Martin Pekar
//  E-Mail  : cortex@nextra.sk
//  Home    : http://www.equinox.szm.sk

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "edeskicon.h"
#include "eiconedit.h"

#include "icons/edit.xpm"
#include "icons/editdelete.xpm"
#include "icons/fileclose.xpm"
#include "icons/filenew.xpm"
#include "icons/fileopen.xpm"


//extern void cbCloseWindow(Fl_Widget*, Fl_Window*);
//void 		icon_edit_dialog(int edit, char* cfg, DeskIcon*w);

void 		DeleteIcon(Fl_Widget*, DeskIcon *);
int 		create_newicon();
void* 		Execute(Fl_Item *, void *);


void		LowerIcon(Fl_Window *, Fl_Window *);
void		CheckConfig(void *);

DeskIcon::
DeskIcon(char *configFile) : Icon (configFile)
{
  LabelWindow = new Fl_Double_Window(0,0,1,1, strdup(icon_name)); 
 // We cannot calculate the size of the label before creating the window,
  LabelWindow->label_size(label_fontsize);			 		   // so set it to arbitrary values here...
  LabelWindow->measure_label(width, height);
    // re-calculate the size of the window based on the space its label needs...
  width = label_maxwidth;
						   // This is a bit strange, but otherwise we get mysterious crashes...
  height= 0;
  fl_measure(icon_name, width, height);
  width += 4;
  height+= 4;
  LabelWindow->position(x_position-((width-32)/2), y_position+34);
  LabelWindow->size(width, height);
  LabelWindow->box(FL_FLAT_BOX);        
  LabelWindow->align(FL_ALIGN_INSIDE | FL_ALIGN_CENTER | FL_ALIGN_WRAP);    
  LabelWindow->color(label_background);
    
  LabelWindow->label_color(label_foreground);
  LabelWindow->clear_border();
  LabelWindow->set_override();
  LabelWindow->end();
  LabelWindow->show();
  this->show();
  SetCursor(this, FL_BLACK, FL_WHITE);
  LowerIcon(this, LabelWindow);
  Fl::add_timeout(1, CheckConfig, (void *)LabelWindow);
  this->begin();
  Fl_Menu_Button *IconPopup = new Fl_Menu_Button(0, 0, 32, 32);
  IconPopup->type(Fl_Menu_Button::POPUP3);
  IconPopup->begin();
    Fl_Item *OpenItem = new Fl_Item("&Open");
    OpenItem->callback((Fl_Callback*)Execute, (void*) this);
    OpenItem->image(new Fl_Pixmap(fileopen_xpm));
    Fl_Item *EditItem = new Fl_Item("&Edit icon");
    EditItem->callback((Fl_Callback*)menuDialogShow, (void*) this);
    EditItem->image(new Fl_Pixmap(edit_xpm));
    Fl_Item *DeleteItem = new Fl_Item("&Delete icon");
    DeleteItem->callback((Fl_Callback*)DeleteIcon, (void*) this);
    DeleteItem->image(new Fl_Pixmap(editdelete_xpm));
    Fl_Item *PropertyItem = new Fl_Item("&Properties");
    PropertyItem->image(new Fl_Pixmap(fileclose_xpm));
  IconPopup->end();
  this->end();	
}

Icon::
Icon(char *icon_config) : Fl_Window(0, 0, 32, 32)
{
  struct stat s;
  char **d = NULL;
  x_position = 100;
  y_position = 100;
  int fd;
  pix = 0;
  mask = 0;
  pix_data = NULL;
  win = 0;
  cfg = icon_config;

  Fl_Config icon_cfg(icon_config);
  icon_cfg.get("Desktop Entry/Icon", icon_file, 4096);
  icon_cfg.get("Desktop Entry/Name", icon_name, 80);
  icon_cfg.get("Desktop Entry/X", x_position);
  icon_cfg.get("Desktop Entry/Y", y_position);
  position(x_position, y_position);
  align(FL_ALIGN_INSIDE | FL_ALIGN_CENTER);
  tooltip(strdup(icon_name));
  end();

  char *iconsDir = PREFIX"/share/ede/pixmaps/32x32/";
  char *iconPath = new char[strlen(icon_file)+strlen(iconsDir)+1];
  sprintf(iconPath, "%s%s", iconsDir, icon_file);
	
  if (!stat(iconPath, &s) && s.st_size > 10) {
	pix_data = (char*) malloc(s.st_size);
    fd = open(iconPath, O_RDONLY);
    if (fd < 0) 
      return;
    read(fd, pix_data, s.st_size);	
    close(fd);
    d = parse_xpm(pix_data, s.st_size);
  }
  delete [] iconPath;
  
  if (d) {
	win = create_window(d, &pix, &mask);
  } 
  else {
	win = create_window(filenew_xpm, &pix, &mask);
  }
}

Icon::~Icon()
{
  free(pix_data);
  XFreePixmap(fl_display, mask);
  XFreePixmap(fl_display, pix);
}

#define EXEC_HELPER "ede-exec"
#define EXEC_OVERHEAD (strlen(EXEC_HELPER)+4)

void*
Execute(Fl_Item *, void *s) 
{
    ((DeskIcon*)s)->execCB();
    return (void*)0;    
}

void DeskIcon::
execCB()
{
    char command[1024];
    Fl_Config flconfig(cfg);
    flconfig.get("Desktop Entry/Exec", command, 1024);
    char *exec_staff = new char[strlen(command)+EXEC_OVERHEAD];    
    
    
    if (strcmp(command,"AddNewIcon") == 0) {
	  create_newicon();
    }
    else {
	sprintf(exec_staff, "%s %s &", EXEC_HELPER, command);
	system(exec_staff);
    }
    delete [] exec_staff;
}

void
LowerIcon(Fl_Window *i, Fl_Window *o)
{
    XLowerWindow(fl_display, fl_xid(i));
    XLowerWindow(fl_display, fl_xid(o));
}

void
raiseIcon(Fl_Window *i, Fl_Window *o)
{
  XRaiseWindow(fl_display, fl_xid(i));
  XRaiseWindow(fl_display, fl_xid(o));
}

int 
DeskIcon::handle(int e)
{
    static int bx, by;
    static int x_icon, x_labeldelta, y_icon, y_labeldelta;
    
    static int button1 = 0;
    int dx, dy;
    
    if (e==FL_PUSH) {
	LowerIcon(this, LabelWindow);
	button1 = (Fl::event_button()==1);
    }
  
    if (button1) 
	switch(e) {
	  case FL_DRAG: {
		//Fl::copy(this->name, strlen(this->name), 1);
	  	//Fl::dnd();
		raiseIcon(this, LabelWindow);
		dx = ((Fl::event_x_root()-bx)/label_gridspacing) * label_gridspacing;
		dy = ((Fl::event_y_root()-by)/label_gridspacing) * label_gridspacing;
        	position(x_icon+dx, y_icon+dy);
		LabelWindow->position(x()+x_labeldelta, y()+y_labeldelta );
		return 1;
	  }
	  case FL_PUSH:
		bx = (Fl::event_x_root()/label_gridspacing)*label_gridspacing;
		by = (Fl::event_y_root()/label_gridspacing)*label_gridspacing;
		x_icon = (x()/label_gridspacing)*label_gridspacing;
		y_icon = (y()/label_gridspacing)*label_gridspacing;

		x_labeldelta = LabelWindow->x()-x();
		y_labeldelta = LabelWindow->y()-y();
		
		LabelWindow->box(FL_BORDER_BOX);
		LabelWindow->redraw();
		
		if (one_click_exec) {
		    execCB();
		}
		else if (Fl::event_clicks() > 0) {
		    Fl::event_clicks(0);	
		    execCB();
		}
    	    return 1;
	}

    switch (e) {
	case FL_RELEASE: {
	  Fl_Config icfg(cfg);
	  icfg.set("Desktop Entry/X", x());
	  icfg.set("Desktop Entry/Y", y());
	  LowerIcon(this, LabelWindow);
	  LabelWindow->box(FL_FLAT_BOX);
	  LabelWindow->redraw();
  	  break;
	}
	case FL_DND_ENTER: { //todo
	  puts("dnd enter.......................................");
  	  return 1;
	}

	case FL_DND_RELEASE: {
	  puts("dnd release.......................................");
  	  return 1;
	}
	case FL_PASTE: {
	  puts("paste.......................................");
	}
	
	case FL_ENTER: {
	  break;
	}
	case FL_LEAVE:	{
	  LowerIcon(this, LabelWindow);
      break;
	}
	default:
	  return Icon::handle(e);
  }	
  return Icon::handle(e);
}

void Icon::
draw()
{
}

void Icon::
show()
{
  return Fl_Window::show();
}

void Icon::
hide()
{
  return Fl_Window::hide();
}

char **Icon::
parse_xpm(char *d, int l)
{
    char **p = NULL;
    int in_quote = 0;
    int r = 0;
    int i = 0;

    while (i < l) { // skip comments
	if (!in_quote && d[i] == '/' && d[i + 1] == '*') {
            i += 2;
            while (i < l) {
                if (d[i] == '*' && d[i + 1] == '/') {
                    i += 2;
                    break;
                }
                i++;
            }
        }

        if (!in_quote && d[i] == '"') {  // string beginning
            p = (char**) realloc(p, sizeof(char*) * (r + 1));
            p[r] = d + i + 1;
            in_quote = 1;
        } 
	else if (in_quote && d[i] == '"') { // string end
            d[i] = '\0';
            in_quote = 0;
        } 
	else if (!in_quote && d[i] == ',') {  // end of line
            r++;
        }
        i++;
    }
    return p;
}

Window Icon::
create_window(char **data, Pixmap *pix, Pixmap *mask)
{
    Window win = 0;
    unsigned long valuemask;
    XSetWindowAttributes attributes;
    XpmAttributes xpm_attributes;
    
    Fl_X::create(this, fl_visual, fl_colormap, -1);
    valuemask = CWEventMask | CWOverrideRedirect | CWSaveUnder;

    attributes.override_redirect = 1;
    attributes.save_under = 1;
    attributes.event_mask = ExposureMask|StructureNotifyMask
	|KeyPressMask|KeyReleaseMask|KeymapStateMask|FocusChangeMask
	|ButtonPressMask|ButtonReleaseMask 
	|EnterWindowMask|LeaveWindowMask
	|PointerMotionMask;
    attributes.colormap = fl_colormap;

    win = fl_xid(this);
    XChangeWindowAttributes(fl_display, win, valuemask, &attributes);
    if (!win) 
	return 0;
	
    xpm_attributes.colormap = fl_colormap;
    xpm_attributes.valuemask = XpmSize | XpmReturnPixels | XpmColormap;
    
    XpmCreatePixmapFromData(fl_display, win, data, pix, mask, &xpm_attributes);
    XShapeCombineMask(fl_display, win, ShapeBounding, 0, 0, *mask, ShapeSet);
    XSetWindowBackgroundPixmap(fl_display, win, *pix);
    XClearWindow(fl_display, win);
    return win;
}

void 
DeskIcon::update_label(char *iConfig)
{
  int width, height;
  Fl_Config iconConfig(cfg);
  iconConfig.get("Desktop Entry/Name", icon_name, 80);
  LabelWindow->begin();
  LabelWindow->measure_label(width, height);
     // re-calculate the size of the window based on the space its label needs...
  width = label_maxwidth;
     // This is a bit strange, but otherwise we get mysterious crashes...
  height= 0;
  fl_measure(icon_name, width, height);
  width += 4;
  height+= 4;
  LabelWindow->position(x()-((width-32)/2), y()+34);
  LabelWindow->size(width, height);

//  delete [] LabelWindow->label();
     // throw away the old label and replace it with the new one
  LabelWindow->copy_label(icon_name);
  LabelWindow->end();
  LabelWindow->redraw();
}

void Icon::update_all(char *icon_config)
{
  struct stat s;
  char **d = NULL;
  char iconFile[1024];
  char *iconName = new char[80];
  int x, y;
  int fd;

    pix = 0;
    mask = 0;
    pix_data = NULL;
    cfg = icon_config;
	
    Fl_Config icon_cfg(icon_config);
    icon_cfg.get("Desktop Entry/Icon", iconFile, 80);
    icon_cfg.get("Desktop Entry/X", x);
    icon_cfg.get("Desktop Entry/Y", y);
    position(x, y);
    icon_cfg.get("Desktop Entry/Name", iconName, 80);
    tooltip(iconName);

    char *iconsDir = PREFIX"/share/ede/pixmaps/32x32/";
    char *iconPath = new char[strlen(iconFile)+strlen(iconsDir)+1];
    sprintf(iconPath, "%s%s", iconsDir, iconFile);
	
    if (!stat(iconPath, &s) && s.st_size > 10) {
	pix_data = (char*) malloc(s.st_size);
	fd = open(iconPath, O_RDONLY);
	if (fd < 0) 
	    return;
	read(fd, pix_data, s.st_size);	
	close(fd);
	d = parse_xpm(pix_data, s.st_size);
    }
    delete [] iconPath;
    if (d) {
        update_icon(d, &pix, &mask);
    } 
    else {
	update_icon(filenew_xpm, &pix, &mask);
    }

}

void Icon::
update_icon(char **data, Pixmap *pix1, Pixmap *mask1)
{
    Window win = 0;
    XpmAttributes xpm_attributes;

    win = fl_xid(this);

    xpm_attributes.colormap = fl_colormap;
    xpm_attributes.valuemask = XpmSize|XpmReturnPixels|XpmColormap;

    XpmCreatePixmapFromData(fl_display,win,data,pix1,mask1,&xpm_attributes);
    XShapeCombineMask(fl_display, win, ShapeBounding, 0, 0, *mask1, ShapeSet);
    XSetWindowBackgroundPixmap(fl_display, win, *pix1);
    XClearWindow(fl_display, win);
}

void 
save_icon(DeskIcon *i_window)
{
  if (strlen(i_name->value())==0) {
	fl_alert("Name of the icon must be filled.");
  }
  else {
    const char *i = i_link->value();
	Fl_Config *i_config = new Fl_Config(i);	
    i_config->set("Desktop Entry/Name", i_name->value());
    i_config->set("Desktop Entry/Exec", i_location->value());
    i_config->set("Desktop Entry/Icon", filename_name(i_filename->value()));
    i_window->update_all((char *)i);
    i_window->update_label((char *)i);
  }	
}

//Fl_Window *edit_window;

void 
DeleteIcon(Fl_Widget*, DeskIcon *i)
{
    if (fl_ask("Delete this icon?")) {
	i->hide();
	i->LabelWindow->hide();
	if (remove(i->GetCfg())<0) 
	    fl_alert("Remove of iconfile %s failed.\nBad access rights to this file.", i->GetCfg());
    }
}

void
update_iconeditdialog(const char *cfg)
{ 
  char val[80], val1[4096], val2[4096];
  i_link->value(cfg);
  Fl_Config i_config(cfg);
  i_config.get("Desktop Entry/Name", val, 80);
  i_name->value(val);
  i_config.get("Desktop Entry/Exec", val1, 4096);
  i_location->value(val1);
  i_config.get("Desktop Entry/Icon", val2, 4096);
  i_filename->value(val2);
}

int 
create_newicon()
{
  const char *i = fl_input("Enter the name of the new icon:", "The Gimp");
  if (i) {
    char *icon_dir = "/.ede/desktop/";
    int size = strlen(getenv("HOME"))+strlen(icon_dir)+strlen(i)+strlen(".desktop")+1;
    char *i_config = new char[size];
    snprintf(i_config, size, "%s%s%s.desktop", getenv("HOME"), icon_dir, i);

	struct stat s;	
	if (stat(i_config, &s)) {
  	  Fl_Config i_cfg(i_config);
	  i_cfg.set("Desktop Entry/Icon", "none");
      i_cfg.set("Desktop Entry/X", 300);
	  i_cfg.set("Desktop Entry/Y", 200);
      i_cfg.set("Desktop Entry/Name", i);

  	  const char *u = fl_input("Program name or location to open:", "gimp");
  	  i_cfg.set("Desktop Entry/Exec", u);
	  
	  DeskIcon *d_icon = new DeskIcon(i_config);  
  	  d_icon->show();
	}
	else {
	  fl_alert("Icon with the same name already exists.");
	}  
  }  
  return 0;
}

inline void DeskIcon::menuDialogShow_i(DeskIcon *o, void *v)
{
  icon_edit_dialog(cfg, (DeskIcon*)v);
}

void DeskIcon::menuDialogShow(DeskIcon *o, void *v)
{
  ((DeskIcon*)(o->user_data()))->menuDialogShow_i(o, v);
}

void 
CheckConfig(void *i) 
{
    conf_clear_cache();
    ReadIconsConfiguration();
    Fl_Double_Window *l = (Fl_Double_Window *)i;
    
    int width, height;
    l->measure_label(width, height);
    width = label_maxwidth;
  
    height= 0;
    fl_measure(l->label(), width, height);
    width += 4;
    height+= 4;
    l->size(width, height);

    l->label_size(label_fontsize);
    l->color(label_background);
    
    l->label_color(label_foreground);
    l->redraw();
    Fl::add_timeout(0.1, CheckConfig, (void*)l);
}

unsigned char 
HAND_BITS[] = 
{
    0x80, 0x01, 0x00, 0x40, 0x02, 0x00, 0x40, 0x02, 0x00, 0x40, 0x02,
    0x00, 0x40, 0x02, 0x00, 0x40, 0x02, 0x00, 0x40, 0x1e, 0x00, 0x40,
    0xf2, 0x00, 0x40, 0x92, 0x01, 0x70, 0x92, 0x02, 0x50, 0x92, 0x04,
    0x48, 0x80, 0x04, 0x48, 0x00, 0x04, 0x48, 0x00, 0x04, 0x08, 0x00,
    0x04, 0x08, 0x00, 0x04, 0x10, 0x00, 0x04, 0x10, 0x00, 0x04, 0x20,
    0x00, 0x02, 0x40, 0x00, 0x02, 0x40, 0x00, 0x01, 0xc0, 0xff, 0x01
};
    
unsigned char 
HAND_MASK_BITS[] = 
{
    0x80, 0x01, 0x00, 0xc0, 0x03, 0x00, 0xc0, 0x03, 0x00, 0xc0, 0x03,
    0x00, 0xc0, 0x03, 0x00, 0xc0, 0x03, 0x00, 0xc0, 0x1f, 0x00, 0xc0,
    0xff, 0x00, 0xc0, 0xff, 0x01, 0xf0, 0xff, 0x03, 0xf0, 0xff, 0x07,
    0xf8, 0xff, 0x07, 0xf8, 0xff, 0x07, 0xf8, 0xff, 0x07, 0xf8, 0xff,
    0x07, 0xf8, 0xff, 0x07, 0xf0, 0xff, 0x07, 0xf0, 0xff, 0x07, 0xe0,
    0xff, 0x03, 0xc0, 0xff, 0x03, 0xc0, 0xff, 0x01, 0xc0, 0xff, 0x01
};

void 
SetCursor(Fl_Window *w, Fl_Color fg, Fl_Color bg) 
{
    Cursor cursor;
    XColor dummy;
    Pixmap p = XCreateBitmapFromData(fl_display,
	RootWindow(fl_display, fl_screen), (const char*)HAND_BITS, 22, 22);
    Pixmap m = XCreateBitmapFromData(fl_display,
	RootWindow(fl_display, fl_screen), (const char*)HAND_MASK_BITS, 22, 22);

    cursor = XCreatePixmapCursor(fl_display, p,m,&dummy, &dummy, 0, 0);
    XFreePixmap(fl_display, m);
    XFreePixmap(fl_display, p);
      
    XColor fgc;
    fg = fl_get_color(fg);
    fgc.red = (fg>>16)&0xFF00;
    fgc.green = (fg>>8)&0xFF00;
    fgc.blue = fg & 0xFF00;
    XColor bgc;
    bg = fl_get_color(bg);
    bgc.red = (bg>>16)&0xFF00;
    bgc.green = (bg>>8)&0xFF00;
    bgc.blue = bg & 0xFF00;
    XRecolorCursor(fl_display, cursor, &fgc, &bgc);
  
    XDefineCursor(fl_display, fl_xid(w), cursor);
}
