//  Volume control for Equinox Desktop Environment
//  Copyright (C) 2000-2001 Martin Pekar
//
//  Requires Fast Light Tool Kit 2.0 (FLTK)
//  http://www.fltk.org - CVS access
//  http://www.agilinux.com/fltk - tar.gz archive
//
//  Author  : Martin Pekar
//  E-Mail  : cortex@nextra.sk
//  Home    : http://www.equinox.szm.sk

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef EVOLCONTROL_H_
#define EVOLCONTROL_H_

extern "C" {
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <linux/soundcard.h>
#include <sys/ioctl.h>
}

#include <fltk/Fl.h>
#include <fltk/Fl_Window.h>
#include <fltk/Fl_Slider.h>
#include <fltk/Fl_Menu_Bar.h>
#include <fltk/Fl_Box.h>
#include <fltk/Fl_Button.h>
#include <fltk/Fl_Item.h>
#include <fltk/Fl_Item_Group.h>
#include <fltk/Fl_Divider.h>
#include <fltk/Fl_Check_Button.h>
#include <fltk/x.h>
#include <fltk/fl_ask.h>
#include <fltk/fl_config.h>

typedef struct 
volume
{
    unsigned char left;
    unsigned char right;

} volume;

int mixer_device;
  
Fl_Slider *volume_slider, *cd_slider, *pcm_slider, *synth_slider,
 *line_slider, *bass_slider, *treble_slider, *mic_slider,
 *speaker_slider, *imix_slider, *igain_slider, *ogain_slider;
   
Fl_Slider *volume_balance, *cd_balance, *pcm_balance, *synth_balance,
 *line_balance, *bass_balance, *treble_balance, *mic_balance,
 *speaker_balance, *imix_balance, *igain_balance, *ogain_balance;
   
Fl_Check_Button *volume_mute, *cd_mute, *pcm_mute, *synth_mute,
 *line_mute, *bass_mute, *treble_mute, *mic_mute,
 *speaker_mute, *imix_mute, *igain_mute, *ogain_mute;

Fl_Check_Button *volume_rec, *cd_rec, *pcm_rec, *synth_rec,
 *line_rec, *bass_rec, *treble_rec, *mic_rec,
 *speaker_rec, *imix_rec, *igain_rec, *ogain_rec;

void get_device_info(int mixer_dev, Fl_Slider *sl, Fl_Slider *bal, Fl_Check_Button *ck, int device);
void set_device(int mixer_fd, int device, Fl_Slider *device_sl, Fl_Slider *balance);
void set_mute(int mixer_fd, int device, Fl_Slider *device_sl, Fl_Slider *balance, Fl_Check_Button *check_button);
void set_rec(int mixer_fd, int device, Fl_Check_Button *ck);
void about_volume_control(Fl_Widget*, void*);

void 
default_look(Fl_Slider* slider) 
{
    slider->type(4);
    slider->minimum(1); 
    slider->maximum(200);
//    slider->maximum(100);    
    slider->value(1);
    slider->step(1); 
    slider->align(FL_ALIGN_TOP);
}

void 
default_look_b(Fl_Slider* balance_slider) 
{
    balance_slider->type(1); 
    balance_slider->minimum(0.00);
    balance_slider->maximum(1.00); 
    balance_slider->step(0.01);
    balance_slider->value(0.01);
}

void 
quit_cb(Fl_Widget*, void*) 
{
    exit(0); 
}

void 
cb_volume(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_VOLUME, volume_slider, volume_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_VOLUME, volume_slider, volume_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_VOLUME, volume_slider, volume_balance, volume_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_VOLUME, volume_rec);
}

void 
cb_cd(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_CD, cd_slider, cd_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_CD, cd_slider, cd_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_CD, cd_slider, cd_balance, cd_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_CD, cd_rec);
}

void 
cb_pcm(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_PCM, pcm_slider, pcm_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_PCM, pcm_slider, pcm_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_PCM, pcm_slider, pcm_balance, pcm_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_PCM, pcm_rec);
}

void 
cb_synth(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_SYNTH, synth_slider, synth_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_SYNTH, synth_slider, synth_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_SYNTH, synth_slider, synth_balance, synth_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_SYNTH, synth_rec);
}

void 
cb_line(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_LINE, line_slider, line_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_LINE, line_slider, line_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_LINE, line_slider, line_balance, line_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_LINE, line_rec);
}


void 
cb_bass(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_BASS, bass_slider, bass_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_BASS, bass_slider, bass_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_BASS, bass_slider, bass_balance, bass_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_BASS, bass_rec);
}

void 
cb_treble(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_TREBLE, treble_slider, treble_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_TREBLE, treble_slider, treble_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_TREBLE, treble_slider, treble_balance, treble_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_TREBLE, treble_rec);
}

void 
cb_mic(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_MIC, mic_slider, mic_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_MIC, mic_slider, mic_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_MIC, mic_slider, mic_balance, mic_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_MIC, mic_rec);
}

void 
cb_speaker(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_SPEAKER, speaker_slider, speaker_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_SPEAKER, speaker_slider, speaker_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_SPEAKER, speaker_slider, speaker_balance, speaker_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_SPEAKER, speaker_rec);
}

void 
cb_imix(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_IMIX, imix_slider, imix_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_IMIX, imix_slider, imix_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_IMIX, imix_slider, imix_balance, imix_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_IMIX, imix_rec);
}

void 
cb_igain(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_IGAIN, igain_slider, igain_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_IGAIN, igain_slider, igain_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_IGAIN, igain_slider, igain_balance, igain_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_IGAIN, igain_rec);
}

void 
cb_ogain(Fl_Slider* o, void *i) 
{
    int x = (int) i;
    if (x == 1) set_device(mixer_device, SOUND_MIXER_OGAIN, ogain_slider, ogain_balance);
    if (x == 2) set_device(mixer_device, SOUND_MIXER_OGAIN, ogain_slider, ogain_balance);
    if (x == 3) set_mute(mixer_device, SOUND_MIXER_OGAIN, ogain_slider, ogain_balance, ogain_mute);
    if (x == 4) set_rec(mixer_device, SOUND_MIXER_OGAIN, ogain_rec);
}

void 
set_device(int 		mixer_fd, 
	   int 		device, 
	   Fl_Slider 	*device_sl, 
	   Fl_Slider 	*balance)
{
//    volume real_volume;
//    real_volume.left = (unsigned int) ((1.0-(balance->value()) ) * device_sl->value() );	
//    real_volume.right = (unsigned int) ( (balance->value()) * device_sl->value());
    int l = (unsigned int) ((1.0-(balance->value()) ) * device_sl->value() );	
    int r = (unsigned int) ( (balance->value()) * device_sl->value());
    
//	printf("volume: %d : %d\n", real_volume.left, real_volume.right);
//	printf("balance: %f\n", (balance->value()));

    int v = (r << 8) | l;

//    if (ioctl (mixer_fd, MIXER_WRITE (device), &real_volume) < 0)
    if (ioctl (mixer_fd, MIXER_WRITE (device), &v) < 0)    
	puts("Cannot set volume, sorry");

}


void 
get_device_info(int 		mixer_dev, 
		Fl_Slider 	*sl, 
		Fl_Slider 	*bal, 
		Fl_Check_Button *ck, 
		int 		device)
{
    unsigned int devmask, recmask, recsrc, stereo;
    volume real_volume;
    
    real_volume.left = real_volume.right = 0;
    devmask = recmask = recsrc = stereo = 0;
    
    if (ioctl(mixer_dev, SOUND_MIXER_READ_DEVMASK, &devmask) == -1) 
	puts("read devmask failed");
    if (devmask & (1 << (device))) 
	sl->activate();
    else 
	sl->deactivate();

    if (ioctl(mixer_dev, SOUND_MIXER_READ_STEREODEVS, &stereo) == -1) printf("read recsrc failed\n"); 
    if ( stereo & (1 << (device) ) ) bal->activate();		    
    else ck->deactivate();		    
		    
    if (ioctl(mixer_dev, SOUND_MIXER_READ_RECMASK, &recmask) == -1) printf("read recmask failed\n");
    if ( recmask & (1 << (device) ) ) ck->activate();		    
    else ck->deactivate();
    
    if (ioctl(mixer_dev, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) printf("read recsrc failed\n"); 
    if ( recsrc & (1 << (device) ) ) ck->set();		    
    else ck->clear();		    
    
    if ( ioctl(mixer_dev, MIXER_READ(device), &real_volume) < 0 ) {
        puts("Can't obtain current volume settings");  
    }

    float volume = real_volume.left + real_volume.right;
//printf("volume: %f\n", volume);

    float balance = 0; 
    balance = ( (1.0 * (unsigned char)real_volume.right ) / 
			(1.0 * ((unsigned char)real_volume.left + (unsigned char)real_volume.right)) );
//printf("balance: %f\n", balance);
    if (volume == 0)
	volume=1;
    if (balance < 0)
	balance=0.5;
    sl->value(volume);
    bal->value(balance);
}



void 
set_mute(int mixer_fd, int device, Fl_Slider *device_sl, Fl_Slider *balance, Fl_Check_Button *check_button)
{
    int vol = 0;
    
    if ( check_button->value() ) {
	if (ioctl(mixer_fd, MIXER_WRITE(device), &vol) < 0 ) 
	 printf("Cannot set mute\n");  
    }
    else {
     volume real_volume;
     double old_volume = device_sl->value();
     double old_balance = balance->value();
     real_volume.left  = (unsigned char) ( (1.0 - (old_balance)) * old_volume );	
     real_volume.right = (unsigned char) ( (old_balance) * old_volume);
     if ( ioctl(mixer_fd, MIXER_WRITE(device), &real_volume) < 0 ){
      printf("Cannot set volume, sorry\n");
     }
    }
}


void 
set_rec (int 			mixer_fd, 
	 int 			device, 
	 Fl_Check_Button 	*ck)
{
    unsigned int recsrc;
    
    if (ioctl(mixer_fd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) printf("read recsrc failed\n");
    unsigned int new_recsrc = recsrc ^ ( 1 << device );
    
    if (ioctl(mixer_fd, SOUND_MIXER_WRITE_RECSRC, &new_recsrc) == -1) printf("oh no\n");				    
}


#endif
