//  File manager for EDE
//  Copyright (C) 2001 Martin Pekar
//
//  Requires Fast Light Tool Kit 2.0 (FLTK)
//  http://www.fltk.org - CVS access
//  http://www.agilinux.com/fltk - tar.gz archive
//
//  Author  : Martin Pekar
//  E-Mail  : cortex@nextra.sk
//  Home    : http://www.equinox.szm.sk

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "dirbrowse.h"

#include "icons/folder.xpm"
#include "icons/editcopy.xpm"
#include "icons/exec.xpm"

extern int io_is_exec (char *);


EFileBrowser::EFileBrowser(int X, int Y, int W, int H, const char*label=0):
    Fl_FileBrowser(X, Y, W, H, label), pType(0), pActive(0) 
{
}

int 
EFileBrowser::panelType(){return pType;}

void 
EFileBrowser::panelType(int pt){pType=pt;}

int 
EFileBrowser::active(){return pActive;}

void 
EFileBrowser::active(int pa){pActive=pa;}

int					// O - Number of files loaded
Fl_FileBrowser::load(const char *directory)// I - Directory to load
{
//puts("overloaded Fl_FileBrowser::load");
  int		i;		// Looping var
  int		num_files;	// Number of files in directory
  char		filename[4096];	// Current file
  Fl_FileIcon	*icon;		// Icon to use
//puts(directory);
//puts("test 1");
  clear();
  directory_ = directory;

//puts("test 1");
  if (directory_[0] == '\0')
  {
    //
    // No directory specified; for UNIX list all mount points.  For DOS
    // list all valid drive letters...
    //

    num_files = 0;
    icon      = Fl_FileIcon::find("any", Fl_FileIcon::DEVICE);

//    if (icon == (Fl_FileIcon *)0)
//      icon = Fl_FileIcon::find("any", Fl_FileIcon::DIR);

#if defined(_WIN32)
    DWORD	drives;		// Drive available bits

    drives = GetLogicalDrives();
    for (i = 'A'; i <= 'Z'; i ++, drives >>= 1)
      if (drives & 1)
      {
        sprintf(filename, "%c:", i);
        add(filename, icon);

	num_files ++;
      }
#elif defined(__EMX__)
    ULONG	curdrive;	// Current drive
    ULONG	drives;		// Drive available bits
    int		start = 3;      // 'C' (MRS - dunno if this is correct!)


    DosQueryCurrentDisk(&curdrive, &drives);
    drives >>= start - 1;
    for (i = 'A'; i <= 'Z'; i ++, drives >>= 1)
      if (drives & 1)
      {
        sprintf(filename, "%c:", i);
        add(filename, icon);

	num_files ++;
      }
#else
//puts("test 1");
    FILE	*mtab;		// /etc/mtab or /etc/mnttab file
    char	line[1024];	// Input line

    //
    // Open the file that contains a list of mounted filesystems...
    //
#  if defined(__hpux) || defined(__sun)
    mtab = fopen("/etc/mnttab", "r");	// Fairly standard
#  elif defined(__sgi) || defined(linux)
    mtab = fopen("/etc/mtab", "r");	// More standard
#  else
    mtab = fopen("/etc/fstab", "r");	// Otherwise fallback to full list
    if (mtab == NULL)
      mtab = fopen("/etc/vfstab", "r");
#  endif
//puts("test 1");
    if (mtab != NULL)
    {
      while (fgets(line, sizeof(line), mtab) != NULL)
      {
        if (line[0] == '#' || line[0] == '\n')
	  continue;
        if (sscanf(line, "%*s%4095s", filename) != 1)
	  continue;

        add(filename, icon);
	num_files ++;
      }

      fclose(mtab);
    }
#endif // _WIN32
  }
  else
  {
    dirent	**files;	// Files in in directory


    //
    // Build the file list...
    //

#if defined(_WIN32) || defined(__EMX__)
    strncpy(filename, directory_, sizeof(filename) - 1);
    filename[sizeof(filename) - 1] = '\0';

    i = strlen(filename) - 1;

    if (i == 2 && filename[1] == ':' &&
        (filename[2] == '/' || filename[2] == '\\'))
      filename[2] = '/';
    else if (filename[i] != '/' && filename[i] != '\\')
      strcat(filename, "/");

    num_files = filename_list(filename, &files);
#else
    num_files = filename_list(directory_, &files);
#endif /* _WIN32 || __EMX__ */

    if (num_files <= 0)
      return (0);
      
    for (i = 0; i < num_files; i ++)     // Add directories first...
      if (strcmp(files[i]->d_name, ".") != 0) {
	snprintf(filename, sizeof(filename), "%s/%s", directory_, files[i]->d_name);

	if (filename_isdir(filename)) {
	    begin();
	    Fl_Item *dirItem = new Fl_Item(strdup(files[i]->d_name));
	    dirItem->box(FL_FLAT_BOX);
	    dirItem->color(FL_WHITE);
	    dirItem->label_color(FL_BLACK);
	    dirItem->label_font(fl_fonts+1);
	    dirItem->image(new Fl_Pixmap(folder_xpm));
	    end();
	  }
      }

    for (i = 0; i < num_files; i ++)
    {
      if (strcmp(files[i]->d_name, ".") != 0) {
	snprintf(filename, sizeof(filename), "%s/%s", directory_, files[i]->d_name);

	if (!filename_isdir(filename) && filename_match(files[i]->d_name, pattern_)) {
	    begin();
	    Fl_Item *fileItem = new Fl_Item(strdup(files[i]->d_name));
	    fileItem->box(FL_FLAT_BOX);
	    fileItem->color(FL_WHITE);
	    fileItem->label_color(FL_BLACK);
	    fileItem->label_font(fl_fonts+1);
	    if (io_is_exec (filename))
		fileItem->image(new Fl_Pixmap(exec_xpm));
	    else
		fileItem->image(new Fl_Pixmap(editcopy_xpm));
	    end();
	}	
      }

      free(files[i]);
    }

    free(files);
  }

  return (num_files);
}

