/* 
 * HardDrake: The Hardware Central Configuration Tool
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 */


#include "libcommon.h"


int irqs[16];
int dmas[8];


/**********************************************************************/
/* HarddrakeDetect: Detect all devices and fille the structure        */
/*                                                                    */
/*            options: harddrake options structure to fill            */
/**********************************************************************/
extern struct harddrake_info
                    *HarddrakeDetect(struct harddrake_options *options){
  int pos = 0;

  struct harddrake_info *result = (struct harddrake_info *)NULL;
  struct harddrake_info *first = (struct harddrake_info *)NULL;

  struct cards_lst *lst = (struct cards_lst *)NULL;
  struct bus_lst *bus = (struct bus_lst*)NULL;	

  struct cpu_info *cpu = (struct cpu_info *)NULL;
  struct memory_info *memory = (struct memory_info *)NULL;
  struct floppy_info *floppy = (struct floppy_info *)NULL;
  struct cdrom_info *cdrom = (struct cdrom_info *)NULL;
  struct disk_info *disk = (struct disk_info *)NULL;
  struct tape_info *tape = (struct tape_info *)NULL;
  struct ethernet_info *ethernet = (struct ethernet_info *)NULL;
  struct soundcard_info *soundcard = (struct soundcard_info *)NULL;
  struct video_info *video = (struct video_info *)NULL;
  struct tvcard_info *tvcard = (struct tvcard_info *)NULL;
  struct mouse_info *mouse = (struct mouse_info *)NULL;
  struct joystick_info *joystick = (struct joystick_info *)NULL;
  struct modem_info *modem = (struct modem_info *)NULL;
  struct printer_info *printer = (struct printer_info *)NULL;
  struct scanner_info *scanner = (struct scanner_info *)NULL;
  struct others_info *others = (struct others_info *)NULL;
  struct identf_info *ideinterface = (struct identf_info *)NULL;
  struct scsintf_info *scsiinterface = (struct scsintf_info *)NULL;
  struct usbntf_info *usbinterface = (struct usbntf_info *)NULL;
  struct isdn_info *isdn = (struct isdn_info *)NULL;
  struct webcam_info *webcam = (struct webcam_info *)NULL;
  struct bridge_info *bridge = (struct bridge_info *)NULL;



  if(first){
    return first;
  }/*endif*/
  /*Detection of all device*/
  lst = init_lst(PATH_ISA_LST, PATH_PCI_LST, PATH_PCMCIA_LST, PATH_USB_LST);
  bus = init_bus(lst);	

    
  /********************************************************************/
  /********************************************************************/

  if(!options->without[CPU]){
    for(cpu = cpu_detect(); cpu; cpu = cpu->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = CPU;
      result->vendor = cpu->vendor;
      result->model = cpu->model;
      result->freq = cpu->freq;
      result->options = cpu->options;
      result->bugs = cpu->bugs;
      result->bogomips = cpu->bogomips;
    }/*next cpu*/
  }/*endif*/
  
  if(!options->without[MEMORY]){
    if((memory = memory_detect())){
      if(!first){
        first = result = malloc_harddrake_info();
      }else {
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = MEMORY;
      result->total = memory->total;
      result->free = memory->free;
      result->shared = memory->shared;
      result->buffers = memory->buffers;
      result->cached = memory->cached;
      result->swap_total = memory->swap_total;
      result->swap_free = memory->swap_free;
    }/*endif*/
  }/*endif*/

  if(!options->without[DISK]){
    for(disk = disk_detect(bus); disk; disk = disk->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = DISK;
      result->bus = disk->bus;
      result->vendor = disk->vendor;
      result->model = disk->model;
      result->device = disk->device;
      result->size = disk->size;
      result->cylinders = disk->cylinders;
      result->heads = disk->heads;
      result->sectors = disk->sectors;
    }/*next disk*/
  }/*endif*/

  if(!options->without[FLOPPY]){
    for(floppy = floppy_detect(bus); floppy; 
        floppy = floppy->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else {
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = FLOPPY;
      result->bus = floppy->bus;
      result->vendor = floppy->vendor;
      result->model = floppy->model;
      result->device = floppy->device;
    }/*next floppy*/
  }/*endif*/

  if(!options->without[CDROM]){
    for (cdrom = cdrom_detect(bus); cdrom; cdrom = cdrom->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = CDROM;
      result->bus = cdrom->bus;
      result->vendor = cdrom->vendor;
      result->model = cdrom->model;
      result->device = cdrom->device;
      result->module = cdrom->module;
    }/*next cdrom*/
  }/*endif*/
  
  if(!options->without[TAPE]){
    for(tape = tape_detect(bus); tape; tape = tape->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = TAPE;
      result->bus = tape->bus;
      result->vendor = tape->vendor;
      result->model = tape->model;
      result->device = tape->device;
    }/*next tape*/
  }/*endif*/

  if(!options->without[ETHERNETCARD]){
    for(ethernet = ethernet_detect(bus), pos = 0; ethernet; 
        ethernet = ethernet->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = ETHERNETCARD;
      result->bus = ethernet->bus;
      result->vendor = ethernet->vendor;
      result->model = ethernet->model;
      result->module = ethernet->module;
      result->pos = pos;
      
      if(ethernet->bus == ISA) {
        result->board_num = ethernet->board_num;
        result->board_id = ethernet->board_id;
        result->dev_num = ethernet->dev_num;
        result->dev_id = ethernet->dev_id;
        result->io = ethernet->io;
        result->irq = ethernet->irq;
      }else{
        result->long_id = ethernet->long_id;
      }/*endif*/
    }/*next ethernet*/
  }/*endif*/
  
  if(!options->without[VIDEOCARD]){
    for(video = video_detect(bus); video; video = video->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = VIDEOCARD;
      result->bus = video->bus;
      result->vendor = video->vendor;
      result->model = video->model;
      // result->module = video->module;
    }/*next video*/
  }/*endif*/
  
  if(!options->without[IDEINTERFACE]){
    for(ideinterface= ideinterface_detect(bus), pos = 0; ideinterface; 
        ideinterface = ideinterface->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = IDEINTERFACE;
      result->bus = ideinterface->bus;
      result->vendor = ideinterface->vendor;
      result->model = ideinterface->model;
      result->module = ideinterface->module;
      
      if(ideinterface->bus == ISA){
        result->board_num = ideinterface->board_num;
        result->board_id = ideinterface->board_id;
        result->dev_num = ideinterface->dev_num;
        result->dev_id = ideinterface->dev_id;
        result->io = ideinterface->io;
        result->irq = ideinterface->irq;
      }else{
        result->long_id = ideinterface->long_id;
      }/*endif*/
    }/*next ideinterface*/
  }/*endif*/
  
  if(!options->without[SCSIINTERFACE]){
    for(scsiinterface= scsiinterface_detect(bus), pos = 0; 
        scsiinterface; scsiinterface = scsiinterface->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = SCSIINTERFACE;
      result->bus = scsiinterface->bus;
      result->vendor = scsiinterface->vendor;
      result->model = scsiinterface->model;
      result->module = scsiinterface->module;
      
      if(scsiinterface->bus == ISA){
        result->board_num = scsiinterface->board_num;
        result->board_id = scsiinterface->board_id;
        result->dev_num = scsiinterface->dev_num;
        result->dev_id = scsiinterface->dev_id;
        result->io = scsiinterface->io;
        result->irq = scsiinterface->irq;
      }else{
        result->long_id = scsiinterface->long_id;
      }/*endif*/
    }/*next scsiinterface*/
  }/*endif*/
  
  if(!options->without[USBINTERFACE]){
    for(pos = 0, usbinterface= usbinterface_detect(bus); usbinterface; 
        usbinterface = usbinterface->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = USBINTERFACE;
      result->bus = usbinterface->bus;
      result->vendor = usbinterface->vendor;
      result->model = usbinterface->model;
      result->module = usbinterface->module;
      result->pos = 0;
      
      if(usbinterface->bus == ISA) {
        result->board_num = usbinterface->board_num;
        result->board_id = usbinterface->board_id;
        result->dev_num = usbinterface->dev_num;
        result->dev_id = usbinterface->dev_id;
        result->io = usbinterface->io;
        result->irq = usbinterface->irq;
      }else{
        result->long_id = usbinterface->long_id;
      }/*endif*/
      
    }/*next usbinterface*/
  }/*endif*/

  if(!options->without[SOUNDCARD]){
    for(soundcard = soundcard_detect(bus), pos = 0; soundcard; 
        soundcard = soundcard->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = SOUNDCARD;
      result->bus = soundcard->bus;
      result->vendor = soundcard->vendor;
      result->model = soundcard->model;
      result->module = soundcard->module;
      result->options = soundcard->options;
      if(soundcard->bus == ISA){
        result->board_num = soundcard->board_num;
        result->board_id = soundcard->board_id;
        result->dev_num = soundcard->dev_num;
        result->dev_id = soundcard->dev_id;
        result->io = soundcard->io;
        result->irq = soundcard->irq;
        result->dma8 = soundcard->dma8;
        if(result->options && HAS_DMA16){
          result->dma16 = soundcard->dma16;
        }/*endif*/
      }else{
        result->long_id = soundcard->long_id;
      }/*endif*/
    }/*next soundcard*/
  }/*endif*/
  
  if(!options->without[TVCARD]){
    for(tvcard = tvcard_detect(bus), pos = 0; 
        tvcard; tvcard = tvcard->next, pos++){ 
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = TVCARD;
      result->bus = tvcard->bus;
      result->vendor = tvcard->vendor;
      result->model = tvcard->model;
      result->module = tvcard->module;
      result->long_id = tvcard->long_id;
    }/*next tvcard*/
  }/*endif*/
  
  if(!options->without[MOUSE]){
    for(mouse = mouse_detect(bus); mouse; 
        mouse = mouse->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = MOUSE;
      result->bus = mouse->bus;
      result->vendor = mouse->vendor;
      result->model = mouse->model;
      result->device = mouse->device;
      result->module = mouse->module;
    }/*next mouse*/
  }/*endif*/

  if(!options->without[JOYSTICK]){
    for(joystick = joystick_detect(bus); joystick; 
        joystick = joystick->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = JOYSTICK;
      result->bus = joystick->bus;
      result->vendor = joystick->vendor;
      result->model = joystick->model;
      result->device = joystick->device;
      result->module = joystick->module;
    }/*next joystick*/
  }/*endif*/

  if(!options->without[PRINTER]){
    for(printer = printer_detect(bus); printer; 
        printer = printer->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = PRINTER;
      result->bus = printer->bus;
      result->vendor = printer->vendor;
      result->model = printer->model;
      result->device = printer->device;
      result->module = printer->module;
    }/*next printer*/
  }/*endif*/
  
  if(!options->without[SCANNER]){
    for (scanner = scanner_detect(bus); scanner; 
         scanner = scanner->next){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = SCANNER;
      result->bus = scanner->bus;
      result->vendor = scanner->vendor;
      result->model = scanner->model;
      result->device = scanner->device;
      result->module = scanner->module;
    }/*next scanner*/
  }/*endif*/
  
  if(!options->without[MODEM]){
    for(modem = modem_detect(bus), pos = 0; modem; 
        modem = modem->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = MODEM;
      result->bus = modem->bus;
      result->vendor = modem->vendor;
      result->model = modem->model;
      result->device = modem->device;
      result->module = modem->module;
      
      if(modem->bus == ISA){
        result->board_num=modem->board_num;
        result->board_id = modem->board_id;
        result->dev_num=modem->dev_num;
        result->dev_id = modem->dev_id;
        result->io = modem->io;
        result->irq = modem->irq;
      }else if(modem->bus == SERIAL){
        result->dev_id = modem->dev_id;
      }else{
        result->long_id = modem->long_id;  
      }/*endif*/
    }/*next modem*/
  }/*endif*/
  
  if(!options->without[OTHERS]){
    for(others = others_detect(bus), pos = 0; others; 
        others = others->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = OTHERS;
      result->bus = others->bus;
      result->vendor = others->vendor;
      result->model = others->model;
      result->device = others->device;
      result->module = others->module;
      
      if(others->bus == ISA){
        result->board_num = others->board_num;
        result->board_id = others->board_id;
        result->dev_num = others->dev_num;
        result->dev_id = others->dev_id;
        result->io = others->io;
        result->irq = others->irq;
      }else if(others->bus == SERIAL){
        result->dev_id = others->dev_id;
      }else{
        result->long_id = others->long_id;  
      }/*endif*/
    }/*next others*/
  }/*endif*/


if(!options->without[ISDN]){
  for(isdn = isdn_detect(bus), pos = 0; isdn; 
        isdn = isdn->next, pos++){
      if(!first){
        first = result = malloc_harddrake_info();
      }else{
        result->next = malloc_harddrake_info();
        result = result->next;
      }/*endif*/
      result->type = ISDN;
      result->bus = isdn->bus;
      result->vendor = isdn->vendor;
      result->model = isdn->model;
      result->module = isdn->module;
      
      if(isdn->bus == ISA){
        result->board_num = isdn->board_num;
        result->board_id = isdn->board_id;
        result->dev_num = isdn->dev_num;
        result->dev_id = isdn->dev_id;
        result->io = isdn->io;
        result->irq = isdn->irq;
      }else if(isdn->bus == SERIAL){
        result->dev_id = isdn->dev_id;
      }else{
        result->long_id = isdn->long_id;  
      }/*endif*/
    }/*next isdn*/
  }/*endif*/


 if(!options->without[WEBCAM]){
   for(webcam = webcam_detect(bus), pos = 0; webcam; 
       webcam = webcam->next, pos++){
     if(!first){
       first = result = malloc_harddrake_info();
     }else{
       result->next = malloc_harddrake_info();
       result = result->next;
     }/*endif*/
     result->type = WEBCAM;
     result->bus = webcam->bus;
     result->vendor = webcam->vendor;
     result->model = webcam->model;
     result->device = webcam->device;
     result->module = webcam->module;
     result->long_id = webcam->long_id;  
   }/*next webcam*/
 }/*endif*/


 if(!options->without[BRIDGE]){
   for(bridge = bridge_detect(bus), pos = 0; bridge; 
       bridge = bridge->next, pos++){
     if(!first){
       first = result = malloc_harddrake_info();
     }else{
       result->next = malloc_harddrake_info();
       result = result->next;
     }/*endif*/
     result->type = BRIDGE;
     result->bus = bridge->bus;
     result->vendor = bridge->vendor;
     result->model = bridge->model;
     result->module = bridge->module;
     result->long_id = bridge->long_id;  
   }/*next bridge*/
 }/*endif*/




  return first;
}/*endfunc HarddrakeDetect*/
