/* 
 * HardDrake: The Hardware Central Configuration Tool
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 */

#ifndef libcommon_H
#define libcommon_H

#include <detect.h>


//#include <config.h>

#include <ctype.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#include <linux/soundcard.h>
#include <sys/ioctl.h>

/*gettext stuff*/
#ifdef ENABLE_NLS
#  include <locale.h>
#    include <libintl.h>
#    define _(String) gettext (String)
#    ifdef gettext_noop
#        define N_(String) gettext_noop (String)
#    else
#        define N_(String) (String)
#    endif
#else
/* Stubs that do something close enough.  */
#    define textdomain(String) (String)
#    define gettext(String) (String)
#    define dgettext(Domain,Message) (Message)
#    define dcgettext(Domain,Message,Type) (Message)  
#    define bindtextdomain(Domain,Directory) (Domain)
#    define _(String) (String)
#    define N_(String) (String)
#endif

#define PATH_ISA_LST PREFIX"/share/detect/isa.lst" 
#define PATH_PCI_LST PREFIX"/share/detect/pci.lst"
#define PATH_PCMCIA_LST PREFIX"/share/detect/pcmcia.lst"
#define PATH_USB_LST PREFIX"/share/detect/usb.lst"


/* IMPORTANT: The following comments are needet, otherwise we won't   */
/* get the strings i18n'ed :-\  (they have to stay AFTER the defines) */

#define MALLOC_ERROR    _("Can't allocate memory!\n")
// _("Can't allocate memory!\n")
#define FOPEN_READ_ERROR        _("Can't open file `%s' for reading!\n")
// _("Can't open file `%s' for reading!\n")
#define FOPEN_WRITE_ERROR       _("Can't open file `%s' for writing!\n")
// _("Can't open file `%s' for writing!\n")


/*System configuration files path*/
#define CONF_MODULES "/etc/conf.modules"
#define TMPCONFMODULE "/tmp/conf.modules"
#define MODULES_CONF "/etc/modules.conf"
#define MODUTILS_CONF "/etc/modutils/Harddrake"
#define PACKAGE "eHardCenter"
#define VERSION "0.6"

#define CONFIG_FILE "/root/.ede/apps/ehardcenter.conf"



/*Modprobe version to be used*/
#define MODPROBE_MAJOR 2
#define MODPROBE_MINOR 3
#define MODPROBE_PATCH 1


#define FREE(p) if (p) free(p);

/*
 * Standard values
 */
extern int standards_io[];
extern int mss_io[];
extern int mpu401_io[];
extern int opl3_io[];
extern int irqs[16];
extern int dmas[8];

/*
 * List of all detect known devices
 */ 
extern DeviceType devtype[];

/*
 * HardDrake options structure
 */
struct harddrake_options {
  int kudzu_mode;
  int timeout;
  int boot;
  int without[30];
  char *cmdconfig[30];
  char *report;
//  GdkColor *fg_color, *bg_color, *unknown_fg_color, *unknown_bg_color;
//  gboolean show_lines;
  char *font;
  char *unknown_font;
};

/*
 * Generic harddrake info
 */
struct harddrake_info {
  char *vendor;                 /* vendor name of the hardware        */
  char *model;                  /* model name of the hardware         */
  char *device;                 /* model name of the hardware         */
  char *module;                 /* name of used kernel module         */
  char *bogomips;
  char *bugs;
  int board_num;
  char *board_id;
  int dev_num;
  char *dev_id;
  unsigned long long_id;
  BusType bus;
  DeviceType type;
  short heads, sectors;
  int cylinders;
  int size;
  int io, mss_io, irq, dma8, dma16, mpu401_io, mpu401_irq, opl3_io;
  int oldset;
  int pos;
  int free;
  int total;
  int shared;
  int buffers;
  int cached;
  int swap_total;
  int swap_free;
  int freq;
  int options;
  int ignore;
  struct harddrake_info *next;
};


/*
 * calls.c: call to external programs
 */
extern int adjust_volume();
extern  void cleanup_mods(DeviceType type, int testmode);
extern char *call_isapnp();
extern char *call_play(char *file);
extern char *call_playmidi(char *file);

/*
 * conv.c: conversions functions
 */
extern char *int2str(int x);
extern char *hex2str(int x);

/*
 * detect.c: detection process functions.
 */
extern struct harddrake_info *HarddrakeDetect(struct harddrake_options *options);


/*
 * options.c: Options handling routines.
 */
extern void usage();
extern struct harddrake_options *InitHarddrake(int argc, char *argv[]);
extern int ReadHarddrakeOptions(struct harddrake_options *options);
extern int WriteHarddrakeOptions(struct harddrake_options *options);

/*
 * report.c: HardDrake report's file handling.
 */
extern struct harddrake_info *ReadHarddrakeReportFile(char *report);
extern int WriteHarddrakeReportFile(struct harddrake_info *hinfo, char *report);

/* 
 * settings.c: configuration files handling routine
 */
extern char *CheckModuleConfPath();
extern int WriteIsaPnPConf(struct harddrake_info *set);
extern int WriteConfModules( struct harddrake_info *set, int testmode);
extern int RemoveConfModules( struct harddrake_info *set);
extern int WriteCardSettings(struct harddrake_info *set);
extern int RemoveCardSettings(struct harddrake_info *set);
extern struct harddrake_info *ReadOldCardSettings();

/*
 * utils.c: Miscenaleounous functions.
 */
extern void *my_malloc1(size_t size);
extern struct harddrake_info *malloc_harddrake_info();
#endif
