/* 
 * HardDrake: The Hardware Central Configuration Tool
 * Copyright (C) 1998-2000 MandrakeSoft
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gnome-xml/parser.h>
#include <gnome-xml/tree.h>

#include "libcommon.h"


#define TRUE 1
#define FALSE 0

/*
 * Important: Position in array must be the same as typedef DevType
 */
DeviceType devtype[] = {
  CPU,
  MEMORY,
  FLOPPY,
  DISK,
  CDROM,
  TAPE,
  ETHERNETCARD,
  MODEM,
  VIDEOCARD,
  TVCARD,
  SOUNDCARD,
  PRINTER,
  SCANNER,
  MOUSE,
  OTHERS,
  IDEINTERFACE,
  SCSIINTERFACE,
  USBINTERFACE,
  JOYSTICK,
  ISDN,
  WEBCAM,
  BRIDGE,
  -1
};


/**********************************************************************/
/* version: Display command line options.                             */
/**********************************************************************/
extern void version() {
  printf("%s %s\t%s\n", PACKAGE, VERSION, _("(c)2000 MandrakeSoft"));
}/*endfunc version*/



/**********************************************************************/
/* usage: print usage informations                                    */
/**********************************************************************/
extern void usage() {
  /* Note to translators: Next strings will be the explanations for   */
  /* the HardDrake commandline options:                               */
  printf("%s\n\n", _("Usage: harddrake [--help] [--report file]"));
  /* --help */
  printf("    --help\t\t\t%s\n", _("Show this help and exit"));
  /* --report <%s>, with %s = `file' */
  printf("    --report <%s>\t\t%s\n", _("file"), _("Use report file <file> to create the tree"));
  /* --version */
  printf("    --version\t\t\t%s\n", _("Show version info and exit"));
  /* --kudzu*/
  printf("    --kudzu\t\t\t%s\n", _("Enable Kudzu mode"));
  /* --boot*/
  printf("    --boot\t\t\t%s\n", _("Enable boot mode (used to configure ISA PnP devices at boot)"));
  /* --timeout sec*/
  printf("    --timeout\t\t\t%s\n", _("Time out (in kudzu mode)"));
  /* --without-cpu */
  printf("    --without-cpu\t\t%s\n", _("Disable cpu detection"));
  /* --without-mem */
  printf("    --without-mem\t\t%s\n", _("Disable memory detection"));
  /* --without-disk */
  printf("    --without-disk\t\t%s\n", _("Disable disk device detection"));
  /* --without-tape */
  printf("    --without-tape\t\t%s\n", _("Disable tape detection"));
  /* --without-floppy */
  printf("    --without-floppy\t\t%s\n", _("Disable floppy device detection"));
  /* --without-cdrom */
  printf("    --without-cdrom\t\t%s\n", _("Disable cdrom detection"));
  /* --without-sccard */
  printf("    --without-sccard\t\t%s\n", _("Disable scsicard detection"));
  /* --without-ethernet */
  printf("    --without-ethernet\t\t%s\n", _("Disable ethernetcard device detection"));
  /* --without-sound */
  printf("    --without-sound\t\t%s\n", _("Disable soundcard device detection"));
  /* --without-mouse */
  printf("    --without-mouse\t\t%s\n", _("Disable mouse device detection"));
  /* --without-modem */
  printf("    --without-modem\t\t%s\n", _("Disable modem device detection"));
  /* --without-printer */
  printf("    --without-printer\t\t%s\n", _("Disable printer device detection"));
  /* --without-ports */
  printf("    --without-ports\t\t%s\n", _("Disable ports detection"));
  /* --without-scanner */
  printf("    --without-scanner\t\t%s\n", _("Disable scanner device detection"));
  /* --without-isdn */
  printf("    --without-isdn\t\t%s\n", _("Disable isdn device detection"));
  /* --without-webcam */
  printf("    --without-webcam\t\t%s\n", _("Disable webcam device detection"));
  /* --without-others */
  printf("    --without-others\t\t%s\n", _("Disable other (uncategorized) device detection"));

}/*endfunc usage*/


/**********************************************************************/
/*InitHardDrake: Fill the HardDrake Options structure                 */
/*                                                                    */
/*       argc: number of args passed to main()                        */
/*       argv: array of args passed to main()                         */
/**********************************************************************/
extern struct harddrake_options *InitHarddrake(int argc, char*argv[]) {
  int i, j;
  char *buff = NULL;
  struct stat *stat_buf = (struct stat *)my_malloc1(sizeof(struct stat));
  
  struct harddrake_options *options = (struct harddrake_options *)
                            my_malloc1(sizeof(struct harddrake_options));
  
  /*Set HardDrake default preferences*/
  options->report = NULL;
  options->kudzu_mode = 0;
  options->timeout = 0;

//  options->show_lines = FALSE;

  /*Set font default*/
  options->font = (char *)my_malloc1(strlen("adobe-helvetica-medium-r-normal-*-*-110-*-*-p-*-iso8859-1, *-r-*"));
  options->unknown_font = (char *)my_malloc1(strlen("adobe-helvetica-bold-r-normal-*-*-110-*-*-p-*-iso8859-1, *-r-*"));

  strcpy(options->font, "-adobe-helvetica-medium-r-normal-*-*-110-*-*-p-*-iso8859-1, *-r-*");
  strcpy(options->unknown_font, "-adobe-helvetica-bold-r-normal-*-*-110-*-*-p-*-iso8859-1, *-r-*");

  /*Set color default*/
/*  options->fg_color = (GdkColor *)my_malloc1(sizeof(GdkColor));
  options->bg_color = (GdkColor *)my_malloc1(sizeof(GdkColor));
  options->unknown_fg_color = (GdkColor *)my_malloc1(sizeof(GdkColor));
  options->unknown_bg_color = (GdkColor *)my_malloc1(sizeof(GdkColor));

  options->unknown_fg_color->red = 0xffff;
  options->unknown_fg_color->green = 0;
  options->unknown_fg_color->blue = 0;
  options->unknown_bg_color->red = 0xffff;
  options->unknown_bg_color->green = 0xffff;
  options->unknown_bg_color->blue = 0xffff;
  options->fg_color->red = 0;
  options->fg_color->green = 0;
  options->fg_color->blue = 0;
  options->bg_color->red = 0xffff;
  options->bg_color->green = 0xffff;
  options->bg_color->blue = 0xffff;*/

  if(!ReadHarddrakeOptions(options)) {
    for(i = 0; devtype[i] != -1; i++){
      options->cmdconfig[i] = NULL;
      options->without[devtype[i]] = 0; 
    }/*next i*/
    if(stat("/usr/sbin/diskdrake", stat_buf) == 0){
      options->cmdconfig[DISK] = "/usr/sbin/diskdrake";
    }/*endif*/
    if(stat("/usr/sbin/diskdrake", stat_buf) == 0){
      options->cmdconfig[MOUSE] = "/usr/sbin/mousedrake";
    }/*endif*/
    if(stat("/usr/sbin/diskdrake", stat_buf) == 0){
      options->cmdconfig[PRINTER] = "/usr/sbin/printerdrake";
    }/*endif*/
  }/*endif*/

  /*Parse command line*/
  for(i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "--report")) {
      if(argv[i + 1]) {
        options->report = (char *)my_malloc1(strlen(argv[i + 1] + 1));
        options->report = argv[i + 1];
      } else {
        printf(_("You haven't passed the detect report file to be used\n"));
        _exit(0);
      }/*endif*/
    }/*endif*/
    
    if(strstr(argv[i], "--without-")) {
      for(j = 0; devtype[j] != -1; j ++) {
        buff = (char *)my_malloc1(strlen("--without-") + strlen(device2str(devtype[j])) + 1);
        sprintf(buff, "--without-%s", device2str(devtype[j]));
        if(!strcmp(argv[i], buff))
          options->without[devtype[j]] = 1;
      }/*next j*/
    }/*endif*/

    if (!strcmp(argv[i], "--timeout")) {
      if(argv[i + 1]) {
        options->timeout = atoi(argv[i + 1]);
      } else {
        printf(_("You haven't passed the timeout delay(in sec)\n"));
        _exit(0);
      }/*endif*/
    }/*endif*/
    
    if(!strcmp(argv[i], "--boot")) {
      options->boot = 1;
    }/*endif*/

    if(!strcmp(argv[i], "--kudzu")) {
      options->kudzu_mode = 1;
    }/*endif*/

    if(!strcmp(argv[i], "--version")) {
      version();
      _exit(0);
    }/*endif*/

    if(!strcmp(argv[i], "--help")) {
      usage();
      _exit(0);
    }/*endif*/
  }/*next i*/

  free(buff);
  
  return options;
}/*endfunc InitHarddrake*/



/**********************************************************************/
/*WriteHarddrakeOptions: Write HardDrake Options to file              */
/*                                                                    */
/*       options: harddrake options structure                         */
/**********************************************************************/

extern int WriteHarddrakeOptions(struct harddrake_options *options) {
  char *filename;
  char buff[128];
  int i;
  
  struct stat s;

  xmlDocPtr old, doc;
  xmlNodePtr tree, subtree;
  xmlNodePtr node;
  
  filename = (char *)my_malloc1(strlen(CONFIG_FILE) + 1);
  strcpy(filename, CONFIG_FILE);
  
#ifdef DEBUG    
  printf("Writing `%s'...\n", filename);
#endif

  if(stat(filename, &s) == -1) {
    printf(_("No configuration file found. Creating a new one...\n"));
    doc = xmlNewDoc("1.0");
    doc->root = xmlNewDocNode(doc, NULL, "HARDDRAKE", NULL);
    xmlSetProp(doc->root, "VERSION", VERSION);
    tree = xmlNewChild(doc->root, NULL, "OPTIONS", NULL);
    tree = xmlNewChild(doc->root, NULL, "DEVICES", NULL);
    xmlSaveFile(filename, doc);
    xmlFreeDoc(doc);
  }

 if(!(old = (xmlDocPtr)xmlParseFile(filename))) {
   fprintf(stderr, FOPEN_WRITE_ERROR, filename);
   return -1;
 }

  doc = xmlNewDoc("1.0");
  doc->root = xmlNewDocNode(doc, NULL, "HARDDRAKE", NULL);
  xmlSetProp(doc->root, "VERSION", VERSION);

  node = old->root->childs;

  while(node) {
    if(!strcmp(node->name, "OPTIONS")) {
      tree = xmlNewChild(doc->root, NULL, "OPTIONS", NULL);
      
      subtree = xmlNewChild(tree, NULL, "KUDZU", NULL);
      if(options->kudzu_mode)
        xmlSetProp(subtree, "ACTIVE", "TRUE");
      else
        xmlSetProp(subtree, "ACTIVE", "FALSE");
      
      sprintf(buff, "%d", options->timeout);
      xmlSetProp(subtree, "TIMEOUT", buff);
      
      subtree = xmlNewChild(tree, NULL, "PREFERENCES", NULL);

      for(i = 0; devtype[i] != -1; i++) {
        subtree = xmlNewChild(tree, NULL, "OPTION", NULL);
        xmlSetProp(subtree, "DEVICE", device2str(devtype[i]));
        xmlSetProp(subtree, "DISABLED", options->without[devtype[i]] == 1 ? "yes" : "no");
        xmlSetProp(subtree, "CMDCONFIG", options->cmdconfig[devtype[i]]);
      }
    } else {
      tree = xmlAddChild(doc->root, node);
    }
    node = node->next;
  }
  
  xmlSaveFile(filename, doc);
  xmlFreeDoc(old);
  free(filename);

  return 0;
}


/**********************************************************************/
/*ReadHardDrakeOptions: Read the HardDrake Options from config. file  */
/*                                                                    */
/*       options:  harddrake options structure                        */
/**********************************************************************/

extern int ReadHarddrakeOptions(struct harddrake_options *options) {
  char *filename;
  char *buff;

  DeviceType type;
 
  xmlDocPtr doc;
  xmlNodePtr node, subnode;


  filename = (char *)my_malloc1(strlen(CONFIG_FILE) + 1);
  strcpy(filename, CONFIG_FILE);

  if(!(doc = (xmlDocPtr)xmlParseFile(filename))) {
#ifdef DEBUG
    printf(_("No configuration file found\n"));
#endif
    return 0;
  }

#ifdef DEBUG
  printf("Reading `%s'...\n", filename);
#endif 

  node = doc->root->childs;

  if(!doc->root || !doc->root->name || strcmp(doc->root->name, "HARDDRAKE") || !doc->root->childs)
     return 0;

  while(node) {
    if(!strcmp(node->name, "OPTIONS")) {
      subnode = node->childs;
      
      if(!subnode)
        return 0;
      
      while(subnode) {
        
        if(!strcmp(subnode->name, "KUDZU")) {
          if((buff = xmlGetProp(subnode, "ACTIVE"))) {
            if(strstr(buff, "TRUE"))
              options->kudzu_mode = 1;
            else
              options->kudzu_mode =0;
          }

          if((buff = xmlGetProp(subnode, "TIMEOUT"))) {
            sscanf(buff,
                   "%d",
                   &options->timeout);
          }
        }
        
        if(!strcmp(subnode->name, "PREFERENCES")) {

        }
        
        if(!strcmp(subnode->name, "OPTION")) {
          type = str2device(xmlGetProp(subnode, "DEVICE"));
          
          if(!strcmp(xmlGetProp(subnode, "DISABLED"), "yes"))
            options->without[type] = 1;
          else options->without[type] = 0;
          
          options->cmdconfig[type] = (char *) my_malloc1(strlen(xmlGetProp(subnode, "CMDCONFIG")) + 1);
          if(strlen(xmlGetProp(subnode, "CMDCONFIG")) > 0)
            strcpy(options->cmdconfig[type], xmlGetProp(subnode, "CMDCONFIG"));
          else
            options->cmdconfig[type] = NULL;
        }
        subnode = subnode->next;
      }
    }
    node = node->next;
  }
  
  free(filename);
  xmlFreeDoc(doc);
  
  return 1;
}/*endfunc ReadHarddrakeOptions*/
