/* 
 * HardDrake: The Hardware Central Configuration Tool
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 */

#include <ctype.h>

#include "libcommon.h"


/**********************************************************************/
/* my_malloc1:  Allocates size bytes and returns a pointer to the      */
/*             allocated memory.  The memory is not cleared.          */
/*                                                                    */
/*     size: Number of bytes ot be allocated.                         */
/**********************************************************************/
extern void *my_malloc1(size_t size){
  void *ptr = NULL;
  
  if(!size){
    return NULL;
  }/*endif*/
  if((ptr = malloc(size)) == NULL){
    fprintf(stderr, MALLOC_ERROR);
    exit(-2);
  }/*endif*/
  return ptr;
}/*endfunc my_malloc1*/


/**********************************************************************/
/* alloc_harddrake_info:  Allocates a HardDrake info structure and    */
/*                        initializes its contents.                   */
/**********************************************************************/
extern struct harddrake_info *malloc_harddrake_info(){
  struct harddrake_info *info;
  
  info = (struct harddrake_info *)
                               my_malloc1(sizeof(struct harddrake_info));
  if(info != (struct harddrake_info *)NULL){
    info->vendor = s_not_available;
    info->model = s_not_available;
    info->device = s_not_available;
    info->module = s_not_available;
    info->bogomips = s_not_available;
    info->bugs = s_not_available;
    info->board_num = 0;
    info->board_id = s_not_available;
    info->dev_num = 0;
    info->dev_id = s_not_available;
    info->long_id = 0;
    info->bus = UNKNOWN_BUS;
    info->type = UNKNOWN_DEVICE;
    info->heads = info->sectors = info->cylinders = 0;
    info->size = 0;
    info->io = info->mss_io = info->irq = -1;
    info->dma8 = info->dma16 = -1;
    info->mpu401_io = info->mpu401_irq = info->opl3_io = -1;
    info->oldset = 0;
    info->pos = 0;
    info->free = 0;
    info->total = 0;
    info->shared = 0;
    info->buffers = 0;
    info->cached = 0;
    info->swap_total = 0;
    info->swap_free = 0;
    info->freq = 0;
    info->options = 0;
    info->ignore = 0;
    info->next = (struct harddrake_info *)NULL;
  }/*endif*/
  return info;
}/*endfunc malloc_harddrake_info*/
