/*
    Scicalc+ : a scientific calculator for X - version 0.86     
    Copyright (C)  2001    Ivano Primi ( ivano.primi@tin.it )    

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    You can contact the author of this software by paper mail writing to
    the next address

	Ivano Primi
	via Colle Cannetacce 50/A
	C.A.P. 00038 - Valmontone (ROMA)
	Italy                                                          .

    If you prefer the electronic mail you can write to the address

	ivano.primi@tin.it                                             .
*/
#include"scicalc.h"

#if (defined(HAVE_UNISTD_H) && defined(HAVE_SYS_TYPES) && defined(HAVE_SYS_STAT) && defined(HAVE_FCNTL_H))

#include<unistd.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<cstdlib>
#include<cstdio>
#include<cstring>
#include<cerrno>

#define DIRNAME_MAXSIZE 1025 // That would be enough
#define MYFILES_MAXSIZE   20 // ....................

/*
struct stat
{
  dev_t         st_dev;       device 
  ino_t         st_ino;       inode 
  mode_t        st_mode;      protection 
  nlink_t       st_nlink;     number of hard links 
  uid_t         st_uid;       user ID of owner 
  gid_t         st_gid;       group ID of owner 
  dev_t         st_rdev;      device type (if inode device) 
  off_t         st_size;      total size, in bytes 
  unsigned long st_blksize;   blocksize for filesystem I/O 
  unsigned long st_blocks;    number of blocks allocated 
  time_t        st_atime;     time of last access 
  time_t        st_mtime;     time of last modification 
  time_t        st_ctime;     time of last change 
};
*/

// The next utility function returns 0 when st1 and st2 are equal,
// !0 when they are different.

#define NO_ERROR 1000000 

static int statcmp(struct stat st1,struct stat st2)
{
  int retval=0;

  retval= (st1.st_dev != st2.st_dev) || (st1.st_ino != st2.st_ino) ;
  retval= retval || (st1.st_mode != st2.st_mode) || (st1.st_nlink != st2.st_nlink);
  retval= retval || (st1.st_uid != st2.st_uid) || (st1.st_gid != st2.st_gid);
  retval= retval || (st1.st_rdev != st2.st_rdev) || (st1.st_size != st2.st_size);
  retval= retval || (st1.st_blksize != st2.st_blksize) || (st1.st_blocks != st2.st_blocks);
  retval= retval || (st1.st_atime != st2.st_atime) || (st1.st_mtime != st2.st_mtime);
  retval= retval || (st1.st_ctime != st2.st_ctime);
  return retval;
}

FILE* open_memofile(int* a)
{
  char cwdname[DIRNAME_MAXSIZE];
  char *memofile;
  int lcwdname,m,mf,fdm,merrno;
  struct stat mstat,mfstat;
  FILE* pm;

  merrno=NO_ERROR;
  if( !getcwd(cwdname,DIRNAME_MAXSIZE) ) // Error in detecting name of
    // the current working directory
    {
      *a=3;
      return NULL;
    }
  cwdname[DIRNAME_MAXSIZE-1]='\0';
  lcwdname=strlen(cwdname);
  memofile=(char*)malloc( (lcwdname+MYFILES_MAXSIZE)*sizeof(char) );
  strcpy(memofile,cwdname);
  memofile[lcwdname]='\0';
  strcat(memofile,"/Memo.mem");
  if( (m=lstat(memofile,&mstat)) )
    merrno=errno;
  fdm=open(memofile,O_RDONLY); // We open just for a try
  if( fdm >= 0 )
    {
      // If the previous condition is true
      // you can call fstat() on *memofile
      mf=fstat(fdm,&mfstat);
      close(fdm);
      if((mf!=m) || statcmp(mstat,mfstat))
	{
	  *a=2;
	  delete memofile;
	  return NULL;
	}
    }
  if( m==0 ) // *memofile exists and can be stated
    {
      long mwrong= mstat.st_mode - (S_IFREG + S_IWUSR + S_IRUSR + S_IWGRP + S_IRGRP + S_IROTH); 

      mwrong= mwrong || (mstat.st_size != MEM_NUMB * sizeof(double));
      *a= (mwrong!=0) ? 2:1;
      if(!mwrong)
	{
	  fdm=open(memofile,O_RDWR);
	  pm=fdopen(fdm,"r+");
	  delete memofile;
	  return pm;
	}
      else
	{
	  delete memofile;
	  return NULL;
	}
    }
  else  // *memofile does not exist or can not be stated
    {
      if(merrno==EBADF || merrno==ELOOP || merrno==EFAULT || merrno==EACCES || merrno==ENOMEM || merrno==ENAMETOOLONG)
	{
	  *a=2;
	  delete memofile;
	  return NULL;
	}
      else // (merrno==NO_ERROR || merrno==ENOENT || merrno==ENOTDIR) && (nerrno==NO_ERROR || nerrno==ENOENT || nerrno==ENOTDIR)
	{
	  *a=0;
	  fdm=open(memofile,O_WRONLY | O_CREAT | O_TRUNC,S_IWUSR | S_IRUSR | S_IWGRP | S_IRGRP | S_IROTH);
	  pm=fdopen(fdm,"w");
	}
    }
  delete memofile;
  return pm;
}

FILE* open_notesfile(int* b)
{
  char cwdname[DIRNAME_MAXSIZE];
  char *notesfile;
  int lcwdname,n,nf,fdn,nerrno;
  struct stat nstat,nfstat;
  FILE* pn;

  nerrno=NO_ERROR;
  if( !getcwd(cwdname,DIRNAME_MAXSIZE) ) // Error in detecting name of
    // the current working directory
    {
      *b=3;
      return NULL;
    }
  cwdname[DIRNAME_MAXSIZE-1]='\0';
  lcwdname=strlen(cwdname);
  notesfile=(char*)malloc((lcwdname+MYFILES_MAXSIZE)*sizeof(char));
  strcpy(notesfile,cwdname);
  notesfile[lcwdname]='\0';
  strcat(notesfile,"/scnotepad");
  if( (n=lstat(notesfile,&nstat)) )
    nerrno=errno;
  fdn=open(notesfile,O_RDONLY); //We open just for a try
  if( fdn >= 0 )
    {
      // If the previous condition is true
      // you can call fstat() on *notesfile
      nf=fstat(fdn,&nfstat);
      close(fdn);
      if((nf!=n) || statcmp(nstat,nfstat))
	{
	  *b=2;
	  delete notesfile;
	  return NULL;
	}
    }
  if( n==0 ) // *notesfile exists and can be stated
    {
      long nwrong= nstat.st_mode - (S_IFREG + S_IWUSR + S_IRUSR + S_IWGRP + S_IRGRP + S_IROTH); 
      
      *b= (nwrong!=0) ? 2:1;
      if(!nwrong)
	{
	  fdn=open(notesfile,O_WRONLY | O_APPEND | O_CREAT);
	  pn=fdopen(fdn,"a");
	  delete notesfile;
	  return pn;
	}
      else
	{
	  delete notesfile;
	  return NULL;
	}
    }
  else // *notesfile does not exist or can not be stated
    {
      if(nerrno==EBADF || nerrno==ELOOP || nerrno==EFAULT || nerrno==EACCES || nerrno==ENOMEM || nerrno==ENAMETOOLONG)
	{
	  *b=2;
	  delete notesfile;
	  return NULL;
	}
      else // (nerrno==NO_ERROR || nerrno==ENOENT || nerrno==ENOTDIR) && (nerrno==NO_ERROR || nerrno==ENOENT || nerrno==ENOTDIR)
	{
	  *b=0;
	  fdn=open(notesfile,O_WRONLY | O_CREAT | O_TRUNC,S_IWUSR | S_IRUSR | S_IWGRP | S_IRGRP | S_IROTH);
	  pn=fdopen(fdn,"w");
	}
    }
  delete notesfile;
  return pn;
}

#else

#include<fltk/filename.h>
#include<cstdio>
#include<cstdlib>
#include<cstring>

static void erase_dirlist(struct dirent** list,int length)
{
  int i;

  for( i = length ; i > 0 ; delete list[--i] )
    ;
  delete list;
}

FILE* open_memofile(int* a)
{
  const char* memofile="Memo.mem";
  FILE *pm;
  int i,retval; // i: counter variable
  struct dirent** dirlist;

  *a=0;
  retval=filename_list(".",&dirlist);
  if(retval<0)
    {
      *a=3;
      return NULL; // Error in scanning the current working directory
    }
  else
    {
      for(i=0 ; i<retval ; i++)
	{
	  if( !strcmp(dirlist[i]->d_name,memofile) )
	    {
	      *a=1; // *memofile already exists
	      if( !(pm=fopen(memofile,"r+b")) )
		{
		  *a=2;
		  erase_dirlist(dirlist,retval);
		  return NULL; // there is some problem with *memofile
		  // (you have no access to it ?)
		}
	      else
		{
		  unsigned long msize;

		  for(msize=0 ; getc(pm) !=EOF ; msize++)
		    ;
		  if((ferror(pm)) || msize!= MEM_NUMB*sizeof(double) || (fseek(pm,0L,SEEK_SET)) )
		    {
		      *a=2;
		      clearerr(pm);
		      fclose(pm);
		      erase_dirlist(dirlist,retval);
		      return NULL;
		    }
		}
	      break;
	    }
	}
      erase_dirlist(dirlist,retval);
      if(*a==0) // *memofile does not exist
	{
	  if( !(pm=fopen(memofile,"wb")) )
	    {
	      *a=2;
	      return NULL; // there is some problem with *memofile
	      // (you have no access to it ?)
	    }
	}
    }//end else
  return pm;
}

FILE* open_notesfile(int* b)
{
  const char* notesfile="scnotepad";  
  FILE *pn;
  int i,retval; 
  struct dirent** dirlist;
  
  *b=0;
  retval=filename_list(".",&dirlist);
  if(retval<0)
    {
      *b=3;
      return NULL; // Error in scanning the current working directory
    }
  else
    {
      for(i=0 ; i<retval ; i++)
	{
	  if( !strcmp(dirlist[i]->d_name,notesfile) )
	    {
	      *b=1;  // *notesfile already exists  
	      if( !(pn=fopen(notesfile,"a")) )
		{
		  *b=2;
		  erase_dirlist(dirlist,retval);
		  return NULL;
		}
	      break;
	    }
	}
      erase_dirlist(dirlist,retval);
      if(*b==0) // *notesfile does not exist
	{
	  if( !(pn=fopen(notesfile,"w")) )
	    {
	      *b=2;
	      return NULL; // there is some problem with *notesfile
	      // (you have no access to it ?)
	    }
	}
    }//end else
  return pn;
}

#endif  
      

