/*
    Scicalc+ : a scientific calculator for X - version 0.86     
    Copyright (C)  2001    Ivano Primi ( ivano.primi@tin.it )    

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    You can contact the author of this software by paper mail writing to
    the next address

	Ivano Primi
	via Colle Cannetacce 50/A
	C.A.P. 00038 - Valmontone (ROMA)
	Italy                                                          .

    If you prefer the electronic mail you can write to the address

	ivano.primi@tin.it                                             .
*/

#include<cstdio>
#include<cstdlib>
#include<new>
#include"scGUI.h"
#include"scicalc.h"

#define STAT1 1
#define STAT2 2

static int statmode=STAT1;
extern double memo[MEM_NUMB];
extern ScInterface* GUI; /* see xscmain.cxx */

static void Stat(Fl_Browser* db, Fl_Browser* rb);

void stat_buttexec_cb( Fl_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;
  Fl_Browser* dbrowser=stat_interface->data_browser;
  Fl_Browser* rbrowser=stat_interface->res_browser;
  int data_size=dbrowser->size();

  rbrowser->clear();
  if(data_size/statmode < 2)
    {
      rbrowser->clear();
      rbrowser->add("Too few data");
      return;
    }
  else if( (statmode==STAT2) && (data_size % 2) )
    {
      rbrowser->clear();
      rbrowser->add("In mode Stat2 the number of data must be even");
      return;
    }
  else
    Stat(dbrowser,rbrowser);
}

void buttstat1_cb(Fl_Check_Button* b, void* v)
{
  statmode=STAT1;
}

void buttstat2_cb(Fl_Check_Button* b, void* v)
{
  statmode=STAT2;
}

void stat_buttnew_cb(Fl_Return_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;
  Fl_Browser* dbrowser=stat_interface->data_browser;
  Fl_Browser* rbrowser=stat_interface->res_browser;
  Fl_Input* in=stat_interface->stat_input;
  const char* newdata;
  int coderr;
  double floatdata;

  newdata=in->value();
  floatdata=vald(newdata,&coderr);
  rbrowser->clear();
  switch(coderr)
    {
    case 1:
      fprintf(stderr,"\a");
      rbrowser->add("Unagreable input");
      return;
    case 2:
      fprintf(stderr,"\a");
      rbrowser->add("Overflow occurred");
      return;
    case FATAL_ERROR:
      fprintf(stderr,"\a");
      fprintf(stderr,"Error in allocation of memory");
      stat_interface->hide();
      GUI->main_win->hide();
      return;
    }
  dbrowser->add(newdata);
  in->value("");
}

void stat_buttdel_cb(Fl_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;
  Fl_Browser* dbrowser=stat_interface->data_browser;
  Fl_Browser* rbrowser=stat_interface->res_browser;
  int selline=dbrowser->value();

  if(!selline)
    {
      fprintf(stderr,"\a");
      rbrowser->clear();
      rbrowser->add("Please, select the line I must erase !");
    }
  else
    dbrowser->remove(selline);
}

void stat_buttins_cb(Fl_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;
  Fl_Browser* dbrowser=stat_interface->data_browser;
  Fl_Browser* rbrowser=stat_interface->res_browser;
  Fl_Input* in=stat_interface->stat_input;
  const char* newdata;
  int coderr,selline=dbrowser->value();
  double floatdata;

  newdata=in->value();
  floatdata=vald(newdata,&coderr);
  rbrowser->clear();
  switch(coderr)
    {
    case 1:
      fprintf(stderr,"\a");
      rbrowser->add("Unagreable input");
      return;
    case 2:
      fprintf(stderr,"\a");
      rbrowser->add("Overflow occurred");
      return;
    case FATAL_ERROR:
      fprintf(stderr,"\a");
      fprintf(stderr,"Error in allocation of memory");
      stat_interface->hide();
      GUI->main_win->hide();
    }
  if(!selline)
    {
      fprintf(stderr,"\a");
      rbrowser->add("Please, select a line first !");
    }
  else
    dbrowser->insert(selline,newdata);
  in->value("");
}

void stat_buttrpl_cb(Fl_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;
  Fl_Browser* dbrowser=stat_interface->data_browser;
  Fl_Browser* rbrowser=stat_interface->res_browser;
  Fl_Input* in=stat_interface->stat_input;
  const char* newdata;
  int coderr,selline=dbrowser->value();
  double floatdata;

  newdata=in->value();
  floatdata=vald(newdata,&coderr);
  rbrowser->clear();
  switch(coderr)
    {
    case 1:
      fprintf(stderr,"\a");
      rbrowser->add("Unagreable input");
      return;
    case 2:
      fprintf(stderr,"\a");
      rbrowser->add("Overflow occurred");
      return;
    case FATAL_ERROR:
      fprintf(stderr,"\a");
      fprintf(stderr,"Error in allocation of memory");
      stat_interface->hide();
      GUI->main_win->hide();
    }
  if(!selline)
    {
      fprintf(stderr,"\a");
      rbrowser->add("Please, select the line I must replace !");
    }
  else
    dbrowser->replace(selline,newdata);
  in->value("");
}

void stat_buttclear_cb(Fl_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;
  Fl_Browser* dbrowser=stat_interface->data_browser;
  Fl_Browser* rbrowser=stat_interface->res_browser;

  dbrowser->clear();
  rbrowser->clear();
}

void stat_buttclose_cb(Fl_Button* b, void* v)
{
  ScStat* stat_interface=(ScStat*)v;

  stat_interface->hide();
}

/*
  La funzione Stat viene utilizzata per l'esecuzione di calcoli statistici.
  Puo' eseguire calcoli statistici su dati suingoli o coppie di dati;
  nel primo caso utilizza,per memorizzare tali dati,il vettore x,nel secon-
  do,i vettori x e y. Poiche' questi hanno una dimensione massima di 100
  non e' possibile lavorare su piu' di 100 dati o coppie di dati.
  Le grandezze statistiche che la funzione e' in grado di calcolare sono
  elencate nel corpo della funzione assieme alle variabili locali  usate
  per la loro memorizzazione. Ogni volta che la funzione viene chiamata
  alcune celle di memoria del vettore memo vengono alterate,in maniera ta-
  le che i risultati dei computi statistici siano utilizzabili dall'utente
  anche dopo l'uscita dalla funzione(cfr. Manuale utente per informazioni
  maggiori).
*/

static void Stat (Fl_Browser* db,Fl_Browser* rb)
{
  double mx, dspx, dscx, sx, sqx, my, dspy, dscy, sy, sqy, sxy, cor, plr, ilr;
  /* mx=media dei valori x
     ** dspx=deviazione standard della popolazione(x)
     ** dscx=deviazione standard del campione(x)
     ** sx=somma dei valori x
     ** sqx=somma dei quadrati dei valori x
     ** my=media dei valori y
     ** dspy=deviazione standard della popolazione y
     ** dscy=deviazione standard del campione y
     ** sy=somma dei valori y
     ** sqy=somma dei quadrati dei valori y
     ** sxy=somma dei prodotti dei valori x e y corrispondenti
     ** cor=coefficiente di correlazione
     ** plr=pendenza linea di regressione
     ** ilr=intercetta linea di regressione
   */
  /* data_size:= numero dei dati */
  /* n:= numero dei dati (STAT1 mode) o delle coppie di dati (STAT2 mode) */
  /* i=  variabile contatore */
  int data_size, n, i;
  /* stringa utilizzata per il check di input numerico */
  const char* sn;
  /* format string */
  char fmtstr[100];
  /* x[] e y[] contengono i valori di tipo x e di tipo y(rispettivamente) */
  double *x, *y;

  data_size=db->size();
  if(statmode==STAT1)
    {
      n=data_size;
      x=new(nothrow)double[n];
      if(!x)
	{
	  fprintf(stderr,"\a");
	  rb->clear();
	  rb->add("Error in allocation of memory");
	  fprintf(stderr,"Error in allocation of memory");
	  /* TO COMPLETE */
	}
      for (i = 0; i < n; i++)
	{
	  sn=db->text(i+1);
	  /* Dal momento che' il controllo su eventuali errori nella    */
	  /* digitazione di numeri viene effettuato nelle funzioni      */
	  /* stat_buttnew_cb, stat_buttins_cb, stat_buttrpl_cb, sn      */
	  /* puo` essere convertita in numero senza alcuna precauzione. */
	  x[i] = atof(sn);
	}
    }
  else
    {
      n=data_size/2;
      x=new(nothrow)double[n];
      y=new(nothrow)double[n];
      if(!x || !y)
	{
	  fprintf(stderr,"\a");
	  rb->clear();
	  rb->add("Error in allocation of memory");
	  fprintf(stderr,"Error in allocation of memory");
	  /* TO COMPLETE */
	}
      for (i = 0; i < n; i++)
	{
	  sn=db->text(2*i+1);
	  /* Dal momento che' il controllo su eventuali errori nella    */
	  /* digitazione di numeri viene effettuato nelle funzioni      */
	  /* stat_buttnew_cb, stat_buttins_cb, stat_buttrpl_cb, sn      */
	  /* puo` essere convertita in numero senza alcuna precauzione. */
	  x[i] = atof (sn);
	  sn=db->text(2*i+2);
	  y[i] = atof (sn);
	}
    }
  if(statmode==STAT1)
    {
      sx = 0;
      sqx = 0;
      for (i = 0; i < n; i++)
	{
	  sx = sx + x[i];
	  sqx = sqx + x[i] * x[i];
	}
      mx = sx / n;
      dspx = 0;
      dscx = 0;
      for (i = 0; i < n; i++)
	dspx = dspx + (x[i] - mx) * (x[i] - mx);
      dscx = sqrt (dspx / (n - 1));
      dspx = sqrt (dspx / n);
      memo[1213] = dspx;  /* memorizzata in ?XP */
      memo[1201] = dscx;  /* memorizzata in ?XD */
      memo[1210] = mx;    /* memorizzata in ?XM */
      snprintf (fmtstr,100,"Sum of the data                =%20.10g\n", sx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Sum of the data squares        =%20.10g\n", sqx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Mean of the data               =%20.10g\n", mx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Population standard deviation  =%20.10g\n", dspx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Standard deviation             =%20.10g\n", dscx);
      rb->add(fmtstr);
      delete[] x;
    }
  else
    {
      sx = 0;
      sqx = 0;
      sy = 0;
      sqy = 0;
      sxy = 0;
      for (i = 0; i < n; i++)
	{
	  sx = sx + x[i];
	  sy = sy + y[i];
	  sqx = sqx + x[i] * x[i];
	  sqy = sqy + y[i] * y[i];
	  sxy = sxy + x[i] * y[i];
	}
      mx = sx / n;
      my = sy / n;
      dspx = 0;
      dscx = 0;
      dspy = 0;
      dscy = 0;
      for (i = 0; i < n; i++)
	{
	  dspx = dspx + (x[i] - mx) * (x[i] - mx);
	  dspy = dspy + (y[i] - my) * (y[i] - my);
	}
      dscx = sqrt (dspx / (n - 1));
      dspx = sqrt (dspx / n);
      dscy = sqrt (dspy / (n - 1));
      dspy = sqrt (dspy / n);
      plr = (n * sxy - sx * sy) / (n * sqx - sx * sx);
      ilr = (sy * sqx - sx * sxy) / (n * sqx - sx * sx);
      cor =
	(n * sxy -
	 sx * sy) / sqrt ((n * sqx - sx * sx) * (n * sqy - sy * sy));
      memo[990]  = ilr;   /* memorizzata in ?RI */
      memo[1000] = plr;   /* memorizzata in ?RS */
      memo[444]  = cor;   /* memorizzata in ?CC */
      memo[1249] = dspy;  /* memorizzata in ?YP */
      memo[1213] = dspx;  /* memorizzata in ?XP */
      memo[1237] = dscy;  /* memorizzata in ?YD */
      memo[1201] = dscx;  /* memorizzata in ?XD */
      memo[1246] = my;    /* memorizzata in ?YM */
      memo[1210] = mx;    /* memorizzata in ?XM */
      snprintf (fmtstr,100,"Sum of the x-data                =%20.10g\n", sx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Sum of the x-data squares        =%20.10g\n", sqx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"X-data mean                      =%20.10g\n", mx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"X-population standard deviation  =%20.10g\n", dspx);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"X-standard deviation             =%20.10g\n", dscx);
      rb->add(fmtstr);
      /* *** */
      snprintf (fmtstr,100,"Sum of the y-data                =%20.10g\n", sy);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Sum of the y-data squares        =%20.10g\n", sqy);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Y-data mean                      =%20.10g\n", my);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Y-population standard deviation  =%20.10g\n", dspy);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Y-standard deviation             =%20.10g\n", dscy);
      rb->add(fmtstr);
      /* *** */
      snprintf (fmtstr,100,"Slope of the regression line     =%20.10g\n", plr);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"X-intersection of the regr. line =%20.10g\n", ilr);
      rb->add(fmtstr);
      snprintf (fmtstr,100,"Correlation constant             =%20.10g\n", cor);
      rb->add(fmtstr);
      delete[] x;
      delete[] y;
    }
}
